/'****************************************************************************
The MIT License

Copyright (c) 2011, Richard D. Clark

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

http://www.opensource.org/licenses/mit-license.php
*****************************************************************************'/
#Include "fbgfx.bi"
#Include "fmod.bi"
#Include "cardobj.bi"
#Include "defs.bi"
#Include "sounds.bi"
#Include "utils.bi"
#Include "cpintro.bi"
#Include "twidgets.bi"

'Init the game.
Sub CreateDeck ()
   Clearkeys
   'Create a deck.
   For i As cards.cardid = cards.cardid.cClubAce To cards.cardid.cSpadKing
      game.stock(i) = i
   Next
   'Shuffle the cards.
   gcards.Shuffle game.stock()
   game.currstock = 52
   If game.soundon = TRUE Then
      'Play shuffle sound.  
      snd.PlayWavSound sndshuffle
   End If
End Sub

'Set up card table.
Sub SetUpTable (fntw As fb.Image Ptr, fntb As fb.Image Ptr)
   Dim As Integer x = 35, y, i, j
   
   'Draw the tableau columns.
   For i = 1 To 7
      x += gcards.CardWidth + 10
      y = 170
      game.tableau(i).fx = x + ((gcards.CardWidth / 2) - 5)
      game.tableau(i).fy = (y - 22)
      Draw String (game.tableau(i).fx + 1, game.tableau(i).fy + 1), Str(i), , fntb
      Draw String (game.tableau(i).fx, game.tableau(i).fy), Str(i), , fntw
      'Draw the tableau rows.
      For j = 1 To 5
         game.tableau(i).currcard = j
         game.tableau(i).cards(j) = game.stock(game.currstock)
         game.tableau(i).coords(j).x = x
         game.tableau(i).coords(j).y = y 
         'game.stock(game.currstock) = cards.cardid.cNone
         game.currstock -= 1
         gcards.DrawCardFront x, y, game.tableau(i).cards(j)
         If game.soundon = TRUE Then
            snd.PlayWavSound snddeal
         End If
         y += 18
      Next
   Next
   'Add card to the foundation.
   game.foundation = game.stock(game.currstock)
   game.fcol = 8
   game.fold = cards.cardid.cNone
   game.fcolold = 0
   game.stock(game.currstock) = cards.cardid.cNone
   game.currstock -= 1
   'Draw the stock.
   x = 324
   y = y + gcards.CardHeight + 30
   gcards.DrawCardBack x, y, cards.cardback.bBlue
   game.cx = x
   game.cy = y
   'Draw stock column number.
   game.sx = x + (gcards.CardWidth / 2) - 5
   game.sy = y - 22
   Draw String (game.sx + 1, game.sy + 1), "8", , fntb
   Draw String (game.sx, game.sy), "8", , fntw
   'Draw the foundation.
   gcards.DrawCardFront x + gcards.CardWidth + 10 , y, game.foundation
   game.fx = x + gcards.CardWidth + 10
   game.fy = y
   'Draw the stock card count.
   x = 290
   y = y + (gcards.CardHeight / 2) - 10
   game.fcx = x
   game.fcy = y
   Draw String (x + 1, y + 1), Str(game.currstock), , fntb
   Draw String (x, y), Str(game.currstock), , fntw
End Sub

Sub Message (st As String, fntw As fb.Image Ptr, fntb As fb.Image Ptr, res As fb.Image Ptr = NULL)
   Dim As Integer x = 19, y = 560
   
   ScreenLock
   'Restore message area.
   If res <> NULL Then
      Put (9, 556), res, PSet
   End If
   Draw String (x + 1, y + 1), st, , fntb
   Draw String (x, y), st, , fntw
   ScreenUnLock
End Sub

'Updates score totals.
Sub UpDateScore (amt As Integer)
   game.score(game.hole) += amt
   game.score(0) += amt
End Sub

Sub DrawScore (fntw As fb.Image Ptr, fntb As fb.Image Ptr)
   Dim As Integer x = 10, y = 40
   Dim As String st
   
   ScreenLock
   st = "Player: " & game.pname
   Draw String (x + 1, y + 1), st, , fntb
   Draw String (x, y), st, , fntw
   y += 22
   st = "Hole: " & game.hole 
   Draw String (x + 1, y + 1), st, , fntb
   Draw String (x, y), st, , fntw
   y += 22
   st = "Score: " & game.score(0)
   Draw String (x + 1, y + 1), st, , fntb
   Draw String (x, y), st, , fntw
   y += 22
   st = "Par: 45"
   Draw String (x + 1, y + 1), st, , fntb
   Draw String (x, y), st, , fntw
   
   ScreenUnLock
End Sub

'Checks column card against foundation card.
Function CheckColumnCard(column As Integer) As cardcompare
   Dim As Integer ccard, fcard
   Dim As cards.cardface cface, fface
   Dim As cardcompare ret = cmpNotOK
   
   'Make sure column still has cards.
   If game.tableau(column).currcard > 0 Then
      'Check the card types for king or ace for wrap around.
      cface = gcards.CFace(game.tableau(column).cards(game.tableau(column).currcard))
      fface = gcards.CFace(game.foundation)
      If (cface = cards.cardface.fKing And fface = cards.cardface.fAce) Or _
         (cface = cards.cardface.fAce And fface = cards.cardface.fKing) Then
         ret = cmpOK
      Else
         'Check the card values.
         ccard = gcards.CValue(game.tableau(column).cards(game.tableau(column).currcard))
         fcard = gcards.CValue(game.foundation)
         'Must be 1 less or 1 greater than foundaton.
         If (ccard - 1 = fcard) Or (ccard + 1 = fcard) Then
            ret = cmpOK
         EndIf
      EndIf
   Else
      ret = cmpEmpty
   EndIf
   
   Return ret
End Function

'Redraws the screen.
Sub DrawMainScreen(fntw As fb.Image Ptr, fntb As fb.Image Ptr, back As fb.image Ptr)
   ScreenLock
   Put (0, 0), back, PSet
   'Redraw the tableau.
   For i As Integer = 1 To 7
      Draw String (game.tableau(i).fx + 1, game.tableau(i).fy + 1), Str(i), , fntb
      Draw String (game.tableau(i).fx, game.tableau(i).fy), Str(i), , fntw
      If game.tableau(i).currcard > 0 Then  
         'Draw the tableau rows.
         For j As Integer = 1 To game.tableau(i).currcard
            gcards.DrawCardFront game.tableau(i).coords(j).x, game.tableau(i).coords(j).y, game.tableau(i).cards(j)
         Next
      End If
   Next
   'Make sure we have cards left in stock.
   If game.currstock > 0 Then
      'Draw stock.
      gcards.DrawCardBack game.cx, game.cy, cards.cardback.bBlue
   Else
      gcards.DrawPlaceholder game.cx, game.cy, cards.cardplace.pRed
   End If
   'Draw stock count.
   Draw String (game.fcx + 1, game.fcy + 1), Str(game.currstock), , fntb
   Draw String (game.fcx, game.fcy), Str(game.currstock), , fntw
   'Draw stock column number.
   Draw String (game.sx + 1, game.sy + 1), "8", , fntb
   Draw String (game.sx, game.sy), "8", , fntw
   'Draw card on foundation
   gcards.DrawCardFront game.fx, game.fy, game.foundation
   'Draw score.
   DrawScore fntw, fntb
   ScreenUnLock
   
End Sub

'Moves a card from column to foundation.
Sub MoveCard(column As Integer, fntw As fb.Image Ptr, fntb As fb.Image Ptr, back As fb.image Ptr)
   'Make sure column has cards.
   If game.tableau(column).currcard > 0 Then
      'Save the old card.
      game.fold = game.foundation
      game.fcolold = game.fcol
      'Get the new card.
      game.foundation = game.tableau(column).cards(game.tableau(column).currcard)
      game.fcol = column
      'game.tableau(column).cards(game.tableau(column).currcard) = cards.cardid.cNone
      game.tableau(column).currcard -= 1
      DrawMainScreen fntw, fntb, back
      If game.soundon = TRUE Then
         snd.PlayWavSound snddeal
      End If
   EndIf
End Sub

'Checks column against foundation.
Sub CheckCard (column As Integer, fntw As fb.Image Ptr, fntb As fb.Image Ptr, back As fb.Image Ptr, mess As fb.Image Ptr)
   Dim As cardcompare chk
   
   chk = CheckColumnCard(column)
   If chk = cmpOK Then
      MoveCard column, fntw, fntb, back
      Message "Press number or click card to move to foundation.", fntw, fntb
   ElseIf chk = cmpEmpty Then
      Message "No cards in column.", fntw, fntb, mess
   Else
      Message "Card must be 1 greater or less than foundation.", fntw, fntb, mess
   EndIf
End Sub

'Checks for remaining moves.
Function CheckForMoves() As Integer
   Dim As Integer ret = FALSE
   Dim As cardcompare chk
   
   For i As Integer = 1 To 7
      chk = CheckColumnCard(i)
      If chk = cmpOK Then
         ret = TRUE
         Exit For
      EndIf
   Next
   
   Return ret
End Function

'Draws stock area.
Sub DrawStock (fntw As fb.Image Ptr, fntb As fb.Image Ptr, back As fb.Image Ptr)
   'Save the current foundation.
   game.fold = game.foundation
   game.fcolold = game.fcol
   'Copy over the stock card.
   game.foundation = game.stock(game.currstock)
   game.fcol = 8
   'Update stock.
   game.currstock -= 1
   DrawMainScreen fntw, fntb, back
   If game.soundon = TRUE Then
      snd.PlayWavSound snddeal
   End If
End Sub

Sub ShowScoreCard ()
   Dim As fb.Image Ptr jana
   Dim As Integer x = 170, y = 48, done = FALSE, tot
   Dim As Double t
   Dim e As fb.event
   Dim As String st
   
   jana = loadbmp("data/jana.bmp")
   ScreenLock
   BLoad "data/scorecard.bmp"
   'Print the name.
   Draw String (x, y), game.pname, , jana 
   For i As Integer = 1 To game.hole
      y += 46
      Draw String (x, y), Str(game.score(i)), , jana
   Next
   y = 508
   tot = game.score(0) - 45
   st = game.score(0) & ", "   
   If tot > 0 Then st &= "+"
   st &= tot
   Draw String (x, y), st, , jana
   ScreenUnLock
   t = Timer
   Do
      If ScreenEvent(@e) Then
         Select Case e.type
            Case fb.EVENT_MOUSE_BUTTON_RELEASE
               done = TRUE
            Case fb.EVENT_KEY_PRESS
               done = TRUE
               Clearkeys
         End Select
      EndIf
      If Timer > t + 10 Then
         done = TRUE
      EndIf
      Sleep 1
   Loop Until done = TRUE
   
End Sub

Sub InitGame (fntw As fb.Image Ptr, fntb As fb.Image Ptr, back As fb.Image Ptr)
   
   ShowHolePic game.hole
   ScreenLock
   Put (0, 0), back, PSet
   ScreenUnLock
   'Get a new deck.
   CreateDeck
   'Set up the table.
   SetUpTable fntw, fntb
   'Draw the score.
   DrawScore fntw, fntb
   'Set initial message.
   Message "Press number or click card to move to foundation.", fntw, fntb
   
End Sub

'Saves game.
function SaveGame (sname As String) As Integer
   Dim As String fname 
   Dim As Integer fh, ret = FALSE
   
   fname = Trim(game.pname) & ".sav"
   fname = LCase(fname)
   fh = FreeFile
   If Open(fname For Binary As #fh) = 0 Then
      Put #fh, ,game
      ret = TRUE
      sname = fname
      Close
   EndIf
   
   Return ret            
End Function

'Toggles sound on/off.
Sub SetSound (fntw As fb.Image Ptr, fntb As fb.Image Ptr, mess As fb.Image Ptr)
   Dim opt As options
   Dim As Integer fh
   Dim As String fname

   If game.soundon = TRUE Then
      game.soundon = FALSE
      Message "Sound is off.", fntw, fntb, mess
   Else
      game.soundon = TRUE
      Message "Sound is on.", fntw, fntb, mess
   End If
   'Save the current options.
   opt.soundon = game.soundon
   opt.musicon = game.musicon
   fname = "golfsol.opt"
   fh = FreeFile
   If Open (fname For Binary As #fh) = 0 Then
      Put #fh,, opt
      Close
   End If
End Sub

'Toggles music on/off.
Sub SetMusic (fntw As fb.Image Ptr, fntb As fb.Image Ptr, mess As fb.Image Ptr)
   Dim opt As options
   Dim As Integer fh
   Dim As String fname
   

   If game.musicon = TRUE Then
      snd.StopSoundTrack
      game.musicon = FALSE
      Message "Music is off.", fntw, fntb, mess
   Else
      snd.PlaySoundTrack("data/music.xm")
      Message "Music is on.", fntw, fntb, mess
      game.musicon = TRUE
   EndIf
   'Save the current options.
   opt.soundon = game.soundon
   opt.musicon = game.musicon
   fname = "golfsol.opt"
   fh = FreeFile
   If Open (fname For Binary As #fh) = 0 Then
      Put #fh,, opt
      Close
   End If
End Sub

'Undos previous move, if any.
Sub Mulligan (fntw As fb.Image Ptr, fntb As fb.Image Ptr, mess As fb.Image Ptr, back As fb.Image Ptr)
   If game.fcolold > 0 Then
      'Check for stock move. 
      If game.fcol = 8 Then
         game.currstock += 1
      Else
         game.tableau(game.fcol).currcard + = 1
      EndIf
      game.fcol = game.fcolold
      game.foundation = game.fold
      game.fcolold = 0
      game.fold = cards.cardid.cNone
      DrawMainScreen fntw, fntb, back
      If game.soundon = TRUE Then
         snd.PlayWavSound snddeal
      End If
      Message "Press number or click card to move to foundation.", fntw, fntb, mess
   Else
      Message "No undo available.", fntw, fntb, mess
   EndIf
End Sub

'Undos previous move, if any.
Sub Reshuffle (fntw As fb.Image Ptr, fntb As fb.Image Ptr, mess As fb.Image Ptr, back As fb.Image Ptr)
   Dim tmp() As cards.cardid
   Dim As Integer cnt, i, j, ti, ci
   
   'Put all the tableau cards into temp array.
   cnt = 0
   For i = 1 To 7
      If game.tableau(i).currcard > 0 Then
         For j = 1 To game.tableau(i).currcard
            cnt += 1
            ReDim Preserve tmp(1 To cnt)
            tmp(cnt) = game.tableau(i).cards(j)
         Next
      End If
   Next
   If game.soundon = TRUE Then
      snd.PlayWavSound sndshuffle
   End If
   'Reshuffle the temp deck.
   gcards.Shuffle tmp()
   'Clear the tableau.
   For i = 1 To 7
      For j = 1 To 5
         game.tableau(i).cards(j) = cards.cardid.cNone
      Next
      game.tableau(i).currcard = 1
   Next
   ti = 1
   ci = 1
   'Reload the tableau.
   For i = 1 To cnt
      game.tableau(ti).cards(ci) = tmp(i)
      game.tableau(ti).currcard = ci
      ti += 1
      If ti > 7 Then
         ti = 1
         ci += 1         
      EndIf
   Next
   'Add one point to current score.
   UpDateScore 1
   'Reset the undo.
   game.fold = cards.cardid.cNone
   game.fcolold = 0
   'Redraw the screen.
   DrawMainScreen fntw, fntb, back
   Message "Tableau reshuffled for 1 point.", fntw, fntb, mess
End Sub


'Shows the high score screen.
Sub AddHighScore ()
   Dim As Integer i, j, fh, fnd, sorted, n, x, y 
   Dim As String fname
   
   fname = "golfsol.hi"
   'Load the high score table.
   If Len(Dir(fname)) > 0 Then
      fh = FreeFile
      If Open(fname For Binary As #fh) = 0 Then
         Get #fh,,his()
         Close
         fnd = FALSE
         'Look for empty spot in array.
         For j = LBound(his) To UBound(his)
            If his(j).used = FALSE Then
               his(j).used = TRUE
               his(j).pname = game.pname
               his(j).total = game.score(0)
               For i = 1 To 9
                  his(j).holes(i) = game.score(i)
               Next
               n = j
               fnd = TRUE
               Exit For
            EndIf
         Next
         If n > 1 And fnd = TRUE Then
            'Sort the table.
            For i = LBound(his) To n - 1
               sorted = TRUE
               For j = LBound(his) To n - 1
                  If his(j).total > his(j + 1).total Then
                     Swap his(j), his(j + 1)
                     sorted = FALSE
                  EndIf
               Next
               If sorted = TRUE Then
                  Exit For
               EndIf
            Next
         End If
         'Replace highest score if higher than current score.
         If fnd = FALSE Then
            n = 0
            For i = LBound(his) To UBound(his)
               If his(i).total > game.score(0) Then
                  n = i
                  Exit For
               EndIf
            Next
            'Move items down.
            If n > 0 Then
               For i = UBound(his) To n + 1 Step -1
                  his(i) = his(i - 1)
               Next
               his(i).used = TRUE
               his(i).pname = game.pname
               his(i).total = game.score(0)
               For j = 1 To 9
                  his(i).holes(j) = game.score(j)
               Next
            EndIf
         EndIf
      Else
         'Set new high score.
         his(1).used = TRUE
         his(1).pname = game.pname
         his(1).total = game.score(0)
         For i = 1 To 9
            his(1).holes(i) = game.score(i)
         Next
      EndIf
   Else
      'Set new high score table.
      his(1).used = TRUE
      his(1).pname = game.pname
      his(1).total = game.score(0)
      For i = 1 To 9
         his(1).holes(i) = game.score(i)
      Next
   EndIf
   'Save the high scores.
   fh = FreeFile
   If Open(fname For Binary As #fh) = 0 Then
      Put #fh,,his()
      Close
   End If
   
End Sub

'Adds game to high score table.
Sub ShowHighScore ()
   Dim As Integer i, j, x, y, fh, fnd, tot
   Dim As String fname, pm
   Dim As fb.Image Ptr fntw, fntb
   
   fntw = loadbmp("data/lucw.bmp")
   fntb = loadbmp("data/lucb.bmp")
   fname = "golfsol.hi"
   fnd = FALSE
   'Load the high score table.
   If Len(Dir(fname)) > 0 Then
      fh = FreeFile
      If Open(fname For Binary As #fh) = 0 Then
         Get #fh,,his()
         Close
         fnd = TRUE
      Else
         fnd = FALSE
      End If
   Else
      fnd = FALSE
   End If
   'Show the screen.
   ScreenLock
   BLoad "data/hiscore.bmp"
   If fnd = TRUE Then
      y = 85
      For i = LBound(his) To UBound(his)
         If his(i).used = TRUE Then
            'Name.
            x = 7
            Draw String (x + 1, y + 1), his(i).pname, , fntb
            Draw String (x, y), his(i).pname, , fntw
            'Holes.
            x = 220
            For j = 1 To 9
               Draw String (x + 1, y + 1), Str(his(i).holes(j)), , fntb
               Draw String (x, y), Str(his(i).holes(j)), , fntw
               x += 39
            Next
            'Total.
            x = 578
            Draw String (x + 1, y + 1), Str(his(i).total), , fntb
            Draw String (x, y), Str(his(i).total), , fntw
            'Plus/minus.
            x = 665
            tot = his(i).total - 45
            If tot > 0 Then
               pm = "+" & tot
            Else
               pm = Str(tot)
            EndIf
            Draw String (x + 1, y + 1), pm, , fntb
            Draw String (x, y), pm, , fntw
            y += 24
         EndIf
      Next
   End If
   ScreenUnLock
   ImageDestroy fntw
   ImageDestroy fntb
   Sleep
   Clearkeys
End Sub

'Main game loop.
Function DoGame (loadgame As Integer = FALSE) As mmenu
   Dim ret As mmenu = mmNone
   Dim As fb.Image Ptr lucw, lucb, messarea, back
   Dim As Integer x, x2, y, y2, done = FALSE, chk, mx, my, fnd, cnt
   Dim e As fb.event
   Dim As String sname
   
   'Set up main screen
   lucw = loadbmp("data/lucw.bmp")
   lucb = loadbmp("data/lucb.bmp")
   back = loadbmp("data/main.bmp")
   Clearkeys
   ClearEvents
   ScreenLock
   Put (0, 0), back, PSet
   ScreenUnLock
   'Save message area.
   messarea = ImageCreate(780, 34)
   Get (9, 556) - (788, 589), messarea
   If loadgame = FALSE Then
      'Set first hole.
      game.hole = 1
      'Clear the score.
      For i As Integer = 0 To 9
         game.score(i) = 0
      Next
      InitGame lucw, lucb, back      
   Else
      DrawMainScreen lucw, lucb, back
   End If
   'Set initial message.
   Message "Press number or click card to move to foundation.", lucw, lucb, messarea
   Do
      If ScreenEvent(@e) Then
         Select Case e.type
            Case fb.EVENT_KEY_PRESS
               'Card columns.
               If e.scancode = fb.SC_ESCAPE Then
                  done = TRUE
               EndIf
               If e.scancode = fb.SC_1 Then
                  CheckCard 1, lucw, lucb, back, messarea
               EndIf
               If e.scancode = fb.SC_2 Then
                  CheckCard 2, lucw, lucb, back, messarea
               EndIf
               If e.scancode = fb.SC_3 Then
                  CheckCard 3, lucw, lucb, back, messarea
               EndIf
               If e.scancode = fb.SC_4 Then
                  CheckCard 4, lucw, lucb, back, messarea
               EndIf
               If e.scancode = fb.SC_5 Then
                  CheckCard 5, lucw, lucb, back, messarea
               EndIf
               If e.scancode = fb.SC_6 Then
                  CheckCard 6, lucw, lucb, back, messarea
               EndIf
               If e.scancode = fb.SC_7 Then
                  CheckCard 7, lucw, lucb, back, messarea
               EndIf
               'Stock.
               If e.scancode = fb.SC_8 Then
                  If game.currstock > 0 Then
                     DrawStock lucw, lucb, back
                     Message "Press number or click card to move to foundation.", lucw, lucb, messarea
                  Else
                     chk = CheckForMoves()
                     If chk = TRUE Then
                        Message "No cards left in stock.", lucw, lucb, messarea
                     End If   
                  EndIf
               EndIf
               'Save game.
               If e.scancode = fb.SC_V Then
                  If SaveGame(sname) = TRUE Then
                     Message "Game saved as " & sname & ".", lucw, lucb, messarea
                  Else
                     Message "Could not save game.", lucw, lucb, messarea
                  EndIf
               EndIf
               'Quit
               If e.scancode = fb.SC_Q Then
                  done = TRUE
               EndIf
               'Sound off on/on.
               If e.scancode = fb.SC_S Then
                  SetSound lucw, lucb, messarea
               EndIf
               'Music off/on.
               If e.scancode = fb.SC_M Then
                  SetMusic lucw, lucb, messarea
               EndIf
               'Mulligan
               If e.scancode = fb.SC_U Then
                  Mulligan lucw, lucb, messarea, back
               EndIf
               'Reshuffle
               If e.scancode = fb.SC_R Then
                  Reshuffle lucw, lucb, messarea, back
               EndIf
               Clearkeys
            Case fb.EVENT_MOUSE_BUTTON_RELEASE
               If e.button = fb.BUTTON_LEFT Then
                  fnd = FALSE
                  'Check each column for card.
                  For i As Integer = 1 To 7
                     If game.tableau(i).currcard > 0 Then
                        x = game.tableau(i).coords(game.tableau(i).currcard).x
                        x2 = (game.tableau(i).coords(game.tableau(i).currcard).x + gcards.CardWidth) - 1
                        y = game.tableau(i).coords(game.tableau(i).currcard).y
                        y2 = (game.tableau(i).coords(game.tableau(i).currcard).y + gcards.CardHeight) - 1
                        If mx >= x And mx <= x2 And my >= y And my <= y2 Then 
                           CheckCard i, lucw, lucb, back, messarea
                           fnd = TRUE
                           Exit For
                        EndIf
                     End If
                  Next
                  'Check for stock card.
                  If fnd = FALSE And game.currstock > 0 Then
                     x = game.cx
                     x2 = (game.cx + gcards.CardWidth) - 1
                     y = game.cy
                     y2 = (game.cy + gcards.CardHeight ) - 1
                     If mx >= x And mx <= x2 And my >= y And my <= y2 Then
                        DrawStock lucw, lucb, back
                        Message "Press number or click card to move to foundation.", lucw, lucb, messarea
                        fnd = TRUE
                     EndIf
                  EndIf
                  'Check for menu items.
                  If fnd = FALSE Then
                     'Reshuffle.
                     If mx >= 11 And mx <= 112 And my >= 2 And my <= 30 Then
                        Reshuffle lucw, lucb, messarea, back
                     EndIf
                     'Mulligan.
                     If mx >= 122 And mx <= 213 And my >= 2 And my <= 30 Then
                        Mulligan lucw, lucb, messarea, back
                     EndIf
                     'Music on/off.
                     If mx >= 232 And mx <= 371 And my >= 2 And my <= 30 Then
                        SetMusic lucw, lucb, messarea
                     EndIf
                     'Sound on/off.
                     If mx >= 382 And mx <= 529 And my >= 2 And my <= 30 Then
                        SetSound lucw, lucb, messarea
                     EndIf
                     'Save game.
                     If mx >= 538 And mx <= 592 And my >= 2 And my <= 30 Then
                        If SaveGame(sname) = TRUE Then
                           Message "Game saved as " & sname & ".", lucw, lucb, messarea
                        Else
                           Message "Could not save game.", lucw, lucb, messarea
                        EndIf
                     EndIf
                     'Quit game.
                     If mx >= 600 And mx <= 654 And my >= 2 And my <= 30 Then
                        done = TRUE
                     EndIf
                  EndIf
               End If
            Case fb.EVENT_MOUSE_MOVE
               mx = e.x
               my = e.y
            Case fb.EVENT_WINDOW_CLOSE
               done = TRUE
               ret = mmQuit
         End Select
      EndIf
      Sleep 1
      'Check for end of round.
      If game.currstock < 1 Then
         chk = CheckForMoves()
         If chk = FALSE Then
            'Add any cards in tableau to score.
            For i As Integer = 1 To 7
               UpDateScore game.tableau(i).currcard
            Next
            'Show score card.
            ShowScoreCard
            game.hole += 1
            If game.hole > 9 Then
               AddHighScore
               ShowHighScore
               done = TRUE
            Else
               InitGame lucw, lucb, back
            EndIf
         EndIf
      Else
         'Check for cleared tableau.
         cnt = 0
         For i As Integer = 1 To 7
            If game.tableau(i).currcard > 0 Then
               cnt += 1
            EndIf
         Next
         'If count is 0, no cards left in tableau.
         If cnt = 0 Then
            'Any cards remaining in stock award a -1.
            UpDateScore -game.currstock
            'Show score card.
            ShowScoreCard
            game.hole += 1
            If game.hole > 9 Then
               AddHighScore
               ShowHighScore
               done = TRUE
            Else
               InitGame lucw, lucb, back
            EndIf
         EndIf
      EndIf
   Loop Until done = TRUE
   'Clear the images.
   ImageDestroy lucw
   ImageDestroy lucb
   ImageDestroy messarea
   ImageDestroy back
   
   Return ret
End Function

'Get player name.
Function GetPlayerName () As Integer
   Dim As tWidgets.btnID btn
   Dim As tWidgets.tInputbox ib
   Dim As Integer ret
   Dim As String pname
   
   ScreenLock
   BLoad "data/bck.bmp"
   ScreenUnLock
   ib.Title = "Player Name"
   ib.Prompt = "Enter your name:"
   ib.MaxLen = 30
   ib.InputLen = 30
   'Get the name of the character.
   btn = ib.Inputbox(pname)
   If btn = tWidgets.btnID.gbnCancel Then
      ret = FALSE
   EndIf
   If btn = tWidgets.btnID.gbnOK Then
      game.pname = pname
      ret = TRUE
   EndIf
   
   Return ret
End Function

'Loads a saved game based on player name.
Function LoadGame(pname As String) As Integer
   Dim As tWidgets.tMsgbox sok
   Dim As tWidgets.btnID btn
   Dim As String fname
   Dim As Integer fh, ret = FALSE
   
   sok.MessageStyle = tWidgets.MsgBoxType.gmbOK
   sok.Title = "Load Game"
   fname = Trim(pname) & ".sav"
   fname = LCase(fname)
   If Len(Dir(fname)) > 0 Then
      fh = FreeFile
      If Open(fname For Binary As #fh) = 0 Then
         Get #fh, ,game
         btn = sok.MessageBox("Game loaded.")
         Close
         ret = TRUE
      Else
         btn = sok.MessageBox("Could not load game.")
      EndIf
   Else
      btn = sok.MessageBox("Save file " & fname & " could be found.")         
   EndIf
      
   Return ret         
End Function

'Loads any saved options.
Sub LoadOptions ()
   Dim opt As options
   Dim As Integer fh
   Dim As String fname
   
   fname = "golfsol.opt"
   fh = FreeFile
   If Len(Dir(fname)) > 0 Then
      If Open (fname For Binary As #fh) = 0 Then
         Get #fh,,opt
         Close
         game.soundon = opt.soundon
         game.musicon = opt.musicon
      Else
         'Set sound flags.
         game.soundon = TRUE
         game.musicon = TRUE
      EndIf
   Else
      'Set sound flags.
      game.soundon = TRUE
      game.musicon = TRUE
   EndIf
   If game.musicon = TRUE Then
      snd.PlaySoundTrack "data/music.xm"
   End If
   
End Sub


'Set to GDI to prevent DX issues on Win32.
#Ifdef __FB_WIN32__
   ScreenControl(fb.SET_DRIVER_NAME, "GDI")
#EndIf
'Set up the screen.
ScreenRes sw, sh, 32
WindowTitle "Golf Solitaire v. " & gsver
Randomize Timer
tWidgets.InitWidgets 'Initialzie the widgets.

LoadOptions

'Show the dev screen.
cproductions.DoCPIntro
'Load card graphics.
Dim As Integer ret
ret = gcards.LoadCards("data/cards.bmp")
If ret = FALSE Then
   Print "Could not load the cards graphic from data folder."
   Sleep
Else
   Dim mret As mmenu
   Do
      'Get the menu selection.
      mret = MainMenu()
      If mret = mmCredits Then
         ShowCredits
      End If
      If mret = mmLoadGame Then
         If GetPlayerName() = TRUE Then
            If LoadGame(game.pname) = TRUE Then
               mret = DoGame(TRUE)
            End If
         End If
      EndIf
      If mret = mmNewGame Then
         If GetPlayerName() = TRUE Then
            mret = DoGame()
         End If
      End If
      If mret = mmHighScores Then
         ShowHighScore
      EndIf
      If mret = mmInstructions Then
         ShowInstructions
      EndIf
      Sleep 1
   Loop Until mret = mmQuit
End If

snd.StopSoundTrack
