/'****************************************************************************
The MIT License

Copyright (c) 2011, Richard D. Clark

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

http://www.opensource.org/licenses/mit-license.php
*****************************************************************************'/

Enum wavids
   sndnone 'no sound
   sndshuffle 'shuffle cards
   snddeal 'Deal sound
   sndgolfswing 'Sound of golf swing
End Enum

'Sound def type
Type sndtype
    channel As Integer
    handle As FSOUND_SAMPLE Ptr
End Type

Type sndsystem
   Private:
   finit As Integer
   soundon As Integer 
   isplaying As Integer 
   sndvol As Integer
   songhandle As FMUSIC_MODULE Ptr
   sndhandle As sndtype
   Public:
   Declare Constructor (svol As Integer = 128)
   Declare Destructor ()
   Declare Property InitOK () As Integer
   Declare Property Sounds (s As Integer)
   Declare Property Sounds () As Integer 
   Declare Property SndVolume () As Integer
   Declare Property MusicIsPlaying () As Integer
   Declare Sub PlaySoundTrack (spec As String = "")
   Declare Sub StopSoundTrack ()
   Declare Sub SetVolume (newvol As Integer)
   Declare Sub PlayWavSound(sndid As wavids, nowait As Integer = FALSE)
   Declare Sub ClearSounds ()
End Type

Constructor sndsystem (svol As Integer = 128)
   sndvol = svol
   soundon = TRUE
   isplaying = FALSE
   songhandle = NULL
   finit = FSOUND_Init(44100, 32, 0)
   If finit Then
     soundon = TRUE
   End If
End Constructor

Destructor sndsystem ()
   StopSoundTrack
End Destructor

Property sndsystem.InitOK () As Integer
   Return finit
End Property

Property sndsystem.Sounds (s As Integer)
   If s = FALSE Then
      soundon = FALSE
   Else
      soundon = TRUE
   EndIf
End Property

Property sndsystem.Sounds () As Integer
   Return soundon
End Property

Property sndsystem.SndVolume () As Integer
   Return sndvol
End Property

Property sndsystem.MusicIsPlaying () As Integer
   Return isplaying
End Property

Sub sndsystem.PlaySoundTrack(sng As String)
   
   If soundon = TRUE Then
      StopSoundTrack
      If Len(Dir(sng)) > 0 Then
         If finit And soundon Then
            songHandle = FMUSIC_LoadSong(sng)
            FMUSIC_SetMasterVolume songHandle, sndvol
            isplaying = FMUSIC_PlaySong(songHandle)
         End If
      End If
   End If
End Sub

Sub sndsystem.StopSoundTrack ()
   Dim As Integer i
   
   If isplaying And songHandle <> NULL Then
      For i = sndvol To 0 Step - 1
         FMUSIC_SetMasterVolume songHandle, i
      Next
      FMUSIC_StopSong songHandle
      FMUSIC_FreeSong songHandle
      isplaying = FALSE
   End If
End Sub

Sub sndsystem.SetVolume(newvol As Integer)
   If newvol < 0 Then 
      sndvol = 0
   ElseIf newvol > 255 Then 
      sndvol = 255
   Else
      sndvol = newvol
   EndIf
   If songhandle <> NULL Then
      FMUSIC_SetMasterVolume songHandle, sndvol
   End If
End Sub

Sub sndsystem.PlayWavSound(sndid As wavids, nowait As Integer = FALSE)
   Dim snd As String
   Dim As Integer done = FALSE
   
   If finit And soundon Then
      Select Case sndid
         Case sndshuffle
             snd = "data/shuffle.wav" 
         Case snddeal
            snd = "data/deal.wav" 
         Case sndgolfswing
            snd = "data/golfswing.wav" 
      End Select
     'Make sure we have a sound and load it
     If snd <> "" Then
        ClearSounds
         If sndhandle.handle = 0 Then
             sndhandle.handle = FSOUND_Sample_Load(FSOUND_FREE, snd, FSOUND_LOOP_OFF, 0, 0)
         End If
         If Not FSOUND_IsPlaying(sndhandle.channel) Then
            sndhandle.channel = FSOUND_PlaySound(FSOUND_FREE, sndhandle.handle)
         End If
         If nowait = FALSE Then
            Do
               If FSOUND_IsPlaying(sndhandle.channel) = FALSE Then
                  done = TRUE
               End If
               Sleep 1
            Loop Until done = TRUE
         End If
     End If
   End If
       
End Sub

Sub sndsystem.ClearSounds ()
   If sndhandle.handle <> NULL Then
      FSOUND_StopSound sndhandle.channel
      FSOUND_Sample_Free sndhandle.handle
      sndhandle.channel = 0
      sndhandle.handle = NULL
   End If
End Sub

Dim Shared snd As sndsystem