/'****************************************************************************
The MIT License

Copyright (c) 2011, Richard D. Clark

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

http://www.opensource.org/licenses/mit-license.php
*****************************************************************************'/

Enum mmenu
   mmNone
   mmNewGame
   mmLoadGame
   mmInstructions
   mmHighScores
   mmCredits
   mmQuit
End Enum


'Used in alpha blend
Type Pixel_Color
    B As UByte
    G As UByte
    R As UByte
    A As UByte
End Type

Union Pixel
    Channel As Pixel_Color
    Value   As UInteger
End Union

Sub Clearkeys ()
   Do While InKey <> ""
      Sleep 1
   Loop 
End Sub

Sub ClearEvents ()
   Dim e As fb.event
   Dim As Integer ret
   
   SetMouse ,,0
   ret = ScreenEvent(@e) 
   Do While ret <> 0
      ret = ScreenEvent(@e)
      Sleep 1
   Loop 
   SetMouse ,,1
End Sub

'Returns an alpha blended color.
Function DoAlphaBlend(Alpha As Integer, fcolor As UInteger, bcolor As UInteger) As UInteger
    Dim As Integer invalpha, r, g, b
    Dim As Pixel fc, bc
    
    If Alpha > 255 Then Alpha = 255
    If Alpha < 0 Then Alpha = 0    
    invalpha = 255 - Alpha
    fc.value = fcolor
    bc.value = bcolor
    r = ((fc.channel.r * Alpha) + (bc.channel.r * invalpha)) Shr 8
    g = ((fc.channel.g * Alpha) + (bc.channel.g * invalpha)) Shr 8
    b = ((fc.channel.b * Alpha) + (bc.channel.b * invalpha)) Shr 8
    
    Return RGB(r, g, b)    
End Function

'' A function that creates an image buffer with the same 
'' dimensions as a BMP image, and loads a file into it.
Function loadbmp( ByRef filename As String ) As Any Ptr

    Dim As Integer filenum, bmpwidth, bmpheight
    Dim As Any Ptr img

    '' open BMP file
    filenum = FreeFile()
    If Open( filename For Binary Access Read As #filenum ) <> 0 Then Return NULL

        '' retrieve BMP dimensions
        Get #filenum, 19, bmpwidth
        Get #filenum, 23, bmpheight

    Close #filenum

    '' create image with BMP dimensions
    img = ImageCreate( bmpwidth, Abs(bmpheight) )

    If img = NULL Then Return NULL

    '' load BMP file into image buffer
    If BLoad( filename, img ) <> 0 Then ImageDestroy( img ): Return NULL

    Return img

End Function

'Main menu
Function MainMenu () As mmenu
   Dim As fb.Image Ptr icon, bck
   Dim e As fb.EVENT
   Dim As mmenu ret = mmNewGame
   Dim As Integer x = 180, y = 140, done = FALSE
   
   Cls
   icon = loadbmp("data/icon.bmp")
   bck = loadbmp("data/menu.bmp")
   If icon = NULL Or bck = NULL Then
      Print "Could not load menu images."
      ret = mmQuit
      Sleep
   Else
      Clearkeys
      ClearEvents
      ScreenLock
      Put (0, 0), bck, PSet
      Put (x, y), icon, Trans
      ScreenUnLock
      Do
         If ScreenEvent(@e) <> 0 Then
            Select Case e.type
               Case fb.EVENT_KEY_PRESS
                  If e.scancode = fb.SC_ESCAPE Then
                     done = TRUE
                     ret = mmQuit
                  End If
                  If e.scancode = fb.SC_UP Then
                     y -= 68
                     ret -= 1
                     If ret < mmNewGame Then
                        ret = mmQuit
                        y = 480
                     EndIf
                     ScreenLock
                     Put (0, 0), bck, PSet
                     Put (x, y), icon, Trans
                     ScreenUnLock
                     If game.soundon = TRUE Then
                        snd.PlayWavSound sndgolfswing, TRUE
                     End If
                  EndIf
                  If e.scancode = fb.SC_DOWN Then
                     y += 68
                     ret += 1
                     If ret > mmQuit Then
                        ret = mmNewGame
                        y = 140
                     EndIf
                     ScreenLock
                     Put (0, 0), bck, PSet
                     Put (x, y), icon, Trans
                     ScreenUnLock
                     If game.soundon = TRUE Then
                        snd.PlayWavSound sndgolfswing, TRUE
                     End If
                  EndIf
                  If e.scancode = fb.SC_ENTER Then
                     done = TRUE
                  EndIf
                  Clearkeys
               Case fb.EVENT_MOUSE_MOVE
                  If e.x >= 226 And e.x <= 577 And e.y >= 130 And e.y <= 184 Then
                     If ret <> mmNewGame Then
                        ret = mmNewGame
                        y = 140
                        ScreenLock
                        Put (0, 0), bck, PSet
                        Put (x, y), icon, Trans
                        ScreenUnLock
                        If game.soundon = TRUE Then
                           snd.PlayWavSound sndgolfswing, TRUE
                        End If
                     End If
                  EndIf
                  If e.x >= 226 And e.x <= 577 And e.y >= 200 And e.y <= 254 Then
                     If ret <> mmLoadGame Then
                        ret = mmLoadGame
                        y = 208 
                        ScreenLock
                        Put (0, 0), bck, PSet
                        Put (x, y), icon, Trans
                        ScreenUnLock
                        If game.soundon = TRUE Then
                           snd.PlayWavSound sndgolfswing, TRUE
                        End If
                     End If
                  EndIf
                  If e.x >= 226 And e.x <= 577 And e.y >= 267 And e.y <= 321 Then
                     If ret <> mmInstructions Then
                        ret = mmInstructions
                        y = 276 
                        ScreenLock
                        Put (0, 0), bck, PSet
                        Put (x, y), icon, Trans
                        ScreenUnLock
                        If game.soundon = TRUE Then
                           snd.PlayWavSound sndgolfswing, TRUE
                        End If
                     End If
                  EndIf
                  If e.x >= 226 And e.x <= 577 And e.y >= 333 And e.y <= 387 Then
                     If ret <> mmHighScores Then
                        ret = mmHighScores
                        y = 344 
                        ScreenLock
                        Put (0, 0), bck, PSet
                        Put (x, y), icon, Trans
                        ScreenUnLock
                        If game.soundon = TRUE Then
                           snd.PlayWavSound sndgolfswing, TRUE
                        End If
                     End If
                  EndIf
                  If e.x >= 226 And e.x <= 577 And e.y >= 402 And e.y <= 456 Then
                     If ret <> mmCredits Then
                        ret = mmCredits
                        y = 412 
                        ScreenLock
                        Put (0, 0), bck, PSet
                        Put (x, y), icon, Trans
                        ScreenUnLock
                        If game.soundon = TRUE Then
                           snd.PlayWavSound sndgolfswing, TRUE
                        End If
                     End If
                  EndIf
                  If e.x >= 226 And e.x <= 577 And e.y >= 472 And e.y <= 526 Then
                     If ret <> mmQuit Then
                        ret = mmQuit
                        y = 480 
                        ScreenLock
                        Put (0, 0), bck, PSet
                        Put (x, y), icon, Trans
                        ScreenUnLock
                        If game.soundon = TRUE Then
                           snd.PlayWavSound sndgolfswing, TRUE
                        End If
                     End If
                  EndIf
               Case fb.EVENT_MOUSE_BUTTON_RELEASE
                  If e.button = fb.BUTTON_LEFT Then
                     done = TRUE
                  End If
               Case fb.EVENT_WINDOW_CLOSE
                  done = TRUE
                  ret = mmQuit
            End Select 
         End If 
         Sleep 1
      Loop Until done = TRUE
   End If
   
   Return ret
End Function

Sub ShowCredits ()
   Dim As Integer done = FALSE
   Dim e As fb.event
   
   
   ScreenLock
   BLoad "data/credits.bmp"
   ScreenUnLock
   Clearkeys
   ClearEvents
   Do
      If ScreenEvent(@e) Then
         Select Case e.type
            Case fb.EVENT_MOUSE_BUTTON_RELEASE
               done = TRUE
            Case fb.EVENT_KEY_PRESS
               done = TRUE
               Clearkeys
         End Select
      EndIf
      Sleep 1
   Loop Until done = TRUE
End Sub

Sub ShowHolePic (holepic As Integer)
   Dim As Integer done = FALSE
   Dim e As fb.event
   Dim As Double t
   
   ScreenLock
   BLoad "data/hole" & Str(holepic) & ".bmp"
   ScreenUnLock
   Clearkeys
   ClearEvents
   t = Timer
   Do
      If ScreenEvent(@e) Then
         Select Case e.type
            Case fb.EVENT_MOUSE_BUTTON_RELEASE
               done = TRUE
            Case fb.EVENT_KEY_PRESS
               done = TRUE
               Clearkeys
         End Select
      EndIf
      If Timer > t + 3 Then
         done = TRUE
      EndIf
      Sleep 1
   Loop Until done = TRUE
   
End Sub

Sub ShowInstructions ()
   Dim As Integer done = FALSE
   Dim e As fb.event
   
   ScreenLock
   BLoad "data/instruct.bmp"
   ScreenUnLock
   Clearkeys
   ClearEvents
   Do
      If ScreenEvent(@e) Then
         Select Case e.type
            Case fb.EVENT_MOUSE_BUTTON_RELEASE
               done = TRUE
            Case fb.EVENT_KEY_PRESS
               done = TRUE
               Clearkeys
         End Select
      EndIf
      Sleep 1
   Loop Until done = TRUE
End Sub
